/*
/* Source code provided freely by Michael Gilleland:
 *
 * http://www.merriampark.com/comb.htm
 *
 */

//--------------------------------------
// Systematically generate combinations.
//--------------------------------------

import java.math.BigInteger;

public class CombinationGenerator {

  private int[] a;
  private int n;
  private int r;
  private BigInteger numLeft;
  private BigInteger total;

  //------------
  // Constructor
  //------------

  public CombinationGenerator (int n, int r) {
    if (r > n) {
      throw new IllegalArgumentException ();
    }
    if (n < 1) {
      throw new IllegalArgumentException ();
    }
    this.n = n;
    this.r = r;
    a = new int[r];
    BigInteger nFact = getFactorial (n);
    BigInteger rFact = getFactorial (r);
    BigInteger nminusrFact = getFactorial (n - r);
    total = nFact.divide (rFact.multiply (nminusrFact));
    reset ();
  }

  //------
  // Reset
  //------

  public void reset () {
    for (int i = 0; i < a.length; i++) {
      a[i] = i;
    }
    numLeft = new BigInteger (total.toString ());
  }

  //------------------------------------------------
  // Return number of combinations not yet generated
  //------------------------------------------------

  public BigInteger getNumLeft () {
    return numLeft;
  }

  //-----------------------------
  // Are there more combinations?
  //-----------------------------

  public boolean hasMore () {
    return numLeft.compareTo (BigInteger.ZERO) == 1;
  }

  //------------------------------------
  // Return total number of combinations
  //------------------------------------

  public BigInteger getTotal () {
    return total;
  }

  //------------------
  // Compute factorial
  //------------------

  private static BigInteger getFactorial (int n) {
    BigInteger fact = BigInteger.ONE;
    for (int i = n; i > 1; i--) {
      fact = fact.multiply (new BigInteger (Integer.toString (i)));
    }
    return fact;
  }

  //--------------------------------------------------------
  // Generate next combination (algorithm from Rosen p. 286)
  //--------------------------------------------------------

  public int[] getNext () {

    if (numLeft.equals (total)) {
      numLeft = numLeft.subtract (BigInteger.ONE);
      return a;
    }

    int i = r - 1;
    while (a[i] == n - r + i) {
      i--;
    }
    a[i] = a[i] + 1;
    for (int j = i + 1; j < r; j++) {
      a[j] = a[i] + j - i;
    }

    numLeft = numLeft.subtract (BigInteger.ONE);
    return a;

  }


  /*
   * This only works for limited values - overflow occurs otherwise
   *   - this is fine for this application - BigInteger is needed for others
   *
   *
   */
  public static int factorial(int val){
      int n = 1;
      
      for (int i=1; i<=val; i++)
         n *= i;

      return n;
  }



  public static int calc_num_combinations(int n, int r){
     // num combinations = (n!)/((r!)(n-r)!)
     return (factorial(n) / (factorial(r) * (factorial(n-r))));
  }






  public static String [] generateCombinations(String [] elements,
                                               int r,
                                               String prefix)
  {
      if (elements == null) {
         System.out.println(" null arg passed to generateCombinations!!\n");
         System.exit(0);
      }

      // Special case when r==0
      // - in our case we actually want to generate one combination for region 0
      if (r == 0) {
         if (prefix != null) {
            String[] combinations = new String[1];
            combinations[0] = prefix;
            return combinations;
         }
         else
            return null;

      }

     int[] indices;
     int num_comb = calc_num_combinations(elements.length, r);

     String[] combinations = new String[num_comb];

     for (int i=0;i<num_comb;i++)
        combinations[i]= prefix;


     CombinationGenerator x = new CombinationGenerator (elements.length, r);

     int j=0;
     while (x.hasMore ()) {
        indices = x.getNext ();
        for (int i = 0; i < indices.length; i++) {
            combinations[j] += elements[indices[i]];
        }
        j++;
     }

//     for (int i=0;i<num_comb; i++)
//         System.out.println(combinations[i]);

     return combinations;
  }


  public static void testCombinations(){

    String[] elements = {"a", "b", "c", "d", "e", "f", "g"};
    int[] indices;

    CombinationGenerator x = new CombinationGenerator (elements.length, 3);
    StringBuffer combination=null;

    while (x.hasMore ()) {
        combination = new StringBuffer ();
        indices = x.getNext ();
        for (int i = 0; i < indices.length; i++) {
            combination.append (elements[indices[i]]);
        }
        System.out.println (combination.toString ());
    }
  }


  public static void main(String[] args) {
      testCombinations();
  }



}









